# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/fewsats-python',
                'doc_host': 'https://Fewsats.github.io',
                'git_url': 'https://github.com/Fewsats/fewsats-python',
                'lib_path': 'fewsats'},
  'syms': { 'fewsats.core': { 'fewsats.core.Fewsats': ('core.html#fewsats', 'fewsats/core.py'),
                              'fewsats.core.Fewsats.__init__': ('core.html#fewsats.__init__', 'fewsats/core.py'),
                              'fewsats.core.Fewsats._preview_payment': ('core.html#fewsats._preview_payment', 'fewsats/core.py'),
                              'fewsats.core.Fewsats._request': ('core.html#fewsats._request', 'fewsats/core.py'),
                              'fewsats.core.Fewsats.as_tools': ('core.html#fewsats.as_tools', 'fewsats/core.py'),
                              'fewsats.core.Fewsats.balance': ('core.html#fewsats.balance', 'fewsats/core.py'),
                              'fewsats.core.Fewsats.create_offers': ('core.html#fewsats.create_offers', 'fewsats/core.py'),
                              'fewsats.core.Fewsats.get_payment_details': ('core.html#fewsats.get_payment_details', 'fewsats/core.py'),
                              'fewsats.core.Fewsats.get_payment_status': ('core.html#fewsats.get_payment_status', 'fewsats/core.py'),
                              'fewsats.core.Fewsats.me': ('core.html#fewsats.me', 'fewsats/core.py'),
                              'fewsats.core.Fewsats.pay_lightning': ('core.html#fewsats.pay_lightning', 'fewsats/core.py'),
                              'fewsats.core.Fewsats.pay_offer': ('core.html#fewsats.pay_offer', 'fewsats/core.py'),
                              'fewsats.core.Fewsats.pay_offer_str': ('core.html#fewsats.pay_offer_str', 'fewsats/core.py'),
                              'fewsats.core.Fewsats.payment_info': ('core.html#fewsats.payment_info', 'fewsats/core.py'),
                              'fewsats.core.Fewsats.payment_methods': ('core.html#fewsats.payment_methods', 'fewsats/core.py'),
                              'fewsats.core.Fewsats.set_webhook': ('core.html#fewsats.set_webhook', 'fewsats/core.py'),
                              'fewsats.core.L402Offers': ('core.html#l402offers', 'fewsats/core.py'),
                              'fewsats.core.L402Offers.__init__': ('core.html#l402offers.__init__', 'fewsats/core.py'),
                              'fewsats.core.L402Offers.__repr__': ('core.html#l402offers.__repr__', 'fewsats/core.py'),
                              'fewsats.core.L402Offers.as_dict': ('core.html#l402offers.as_dict', 'fewsats/core.py'),
                              'fewsats.core.L402Offers.from_dict': ('core.html#l402offers.from_dict', 'fewsats/core.py'),
                              'fewsats.core.Offer': ('core.html#offer', 'fewsats/core.py'),
                              'fewsats.core.Offer.__init__': ('core.html#offer.__init__', 'fewsats/core.py'),
                              'fewsats.core.Offer.__repr__': ('core.html#offer.__repr__', 'fewsats/core.py'),
                              'fewsats.core.Offer.from_dict': ('core.html#offer.from_dict', 'fewsats/core.py')},
            'fewsats.l402': { 'fewsats.l402.L.starstarmap': ('claudette.html#l.starstarmap', 'fewsats/l402.py'),
                              'fewsats.l402.Offer': ('claudette.html#offer', 'fewsats/l402.py'),
                              'fewsats.l402.Offer.__init__': ('claudette.html#offer.__init__', 'fewsats/l402.py'),
                              'fewsats.l402.Offer.supports': ('claudette.html#offer.supports', 'fewsats/l402.py'),
                              'fewsats.l402.PaymentInfo': ('claudette.html#paymentinfo', 'fewsats/l402.py'),
                              'fewsats.l402.PaymentInfo.__init__': ('claudette.html#paymentinfo.__init__', 'fewsats/l402.py'),
                              'fewsats.l402.PaymentOffers': ('claudette.html#paymentoffers', 'fewsats/l402.py'),
                              'fewsats.l402.PaymentOffers.__init__': ('claudette.html#paymentoffers.__init__', 'fewsats/l402.py'),
                              'fewsats.l402.PaymentOffers.__str__': ('claudette.html#paymentoffers.__str__', 'fewsats/l402.py'),
                              'fewsats.l402.PaymentOffers.by_payment': ('claudette.html#paymentoffers.by_payment', 'fewsats/l402.py'),
                              'fewsats.l402.get_payment_details': ('claudette.html#get_payment_details', 'fewsats/l402.py')}}}



"""Utility functions for interacting with L402 servers"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/02_claudette.ipynb.

# %% auto 0
__all__ = ['Offer', 'PaymentOffers', 'PaymentInfo', 'get_payment_details']

# %% ../nbs/02_claudette.ipynb 2
from fastcore.utils import L, first, last, groupby, mul, add, sub, truediv, patch, store_attr, BasicRepr
import fastcore.all as fc
import os
from collections.abc import Iterable
import httpx

# %% ../nbs/02_claudette.ipynb 22
class Offer(fc.BasicRepr):
    def __init__(self, offer_id, title, description, type, balance, amount, currency, payment_methods):
        store_attr()

# %% ../nbs/02_claudette.ipynb 26
@patch
def starstarmap(self:L, f): return L(f(**o) for o in self)

# %% ../nbs/02_claudette.ipynb 30
@patch
def supports(self: Offer, payment): return payment in self.payment_methods

# %% ../nbs/02_claudette.ipynb 35
class PaymentOffers(fc.BasicRepr):
    def __init__(self, *, version, offers, payment_request_url, payment_context_token, terms_url, metadata=None, expiry=None):
        self.version, self.payment_request_url, self.payment_context_token, self.terms_url, self.metadata, self.offers = (
            version, payment_request_url, payment_context_token, terms_url, metadata, L(offers).starstarmap(Offer))

# %% ../nbs/02_claudette.ipynb 37
@patch
def __str__(self: PaymentOffers):
    return (f"L402 Payment Offers (v{self.version})\n" + 
            '\n'.join(f"- {o.offer_id} '{o.title}' ${o.amount}, {o.balance}, {o.payment_methods}" 
                     for o in self.offers))

# %% ../nbs/02_claudette.ipynb 43
@patch
def by_payment(self:PaymentOffers, payment_type, reverse=False):
    'Returns offers matching the `payment_type` sortedy by `amount`'
    return self.offers.filter(lambda o: o.supports(payment_type)).sorted(lambda o: o.amount, reverse=reverse)

# %% ../nbs/02_claudette.ipynb 58
class PaymentInfo(fc.BasicRepr):
    def __init__(self, *, offer_id, expires_at, payment_request, version):
        store_attr()


# %% ../nbs/02_claudette.ipynb 60
def get_payment_details(url, oid, pm, t):
    "Get payment details for an offer using offer id `oid` and payment method `pm`"
    return PaymentInfo(**httpx.post(url, json={'offer_id': oid, 'payment_method': pm, 'payment_context_token': t}).json())



"""Client for interacting with the Fewsats API"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['Fewsats', 'Offer', 'L402Offers']

# %% ../nbs/00_core.ipynb 2
from fastcore.utils import *
import os
import httpx
from typing import Dict, Any, List
import json
from fastcore.basics import BasicRepr
from fastcore.utils import store_attr
from typing import List, Dict, Any

# %% ../nbs/00_core.ipynb 6
class Fewsats:
    "Client for interacting with the Fewsats API"
    def __init__(self,
                 api_key: str = None, # The API key for the Fewsats account
                 base_url: str = "https://api.fewsats.com"): # The Fewsats API base URL
        self.api_key = api_key or os.environ.get("FEWSATS_API_KEY")
        if not self.api_key:
            raise ValueError("The api_key client option must be set either by passing api_key to the client or by setting the FEWSATS_API_KEY environment variable")
        self.base_url = base_url
        self._httpx_client = httpx.Client()
        self._httpx_client.headers.update({"Authorization": f"Token {self.api_key}"})


# %% ../nbs/00_core.ipynb 9
@patch
def _request(self: Fewsats, 
             method: str, # The HTTP method to use
             path: str, # The path to request
             timeout: int = 10, # Timeout for the request in s
             **kwargs) -> Dict[str, Any]:
    "Makes an authenticated request to Fewsats API"
    url = f"{self.base_url}/{path}"
    return  self._httpx_client.request(method, url, timeout=timeout, **kwargs)

# %% ../nbs/00_core.ipynb 13
@patch
def me(self: Fewsats):
    "Retrieve the user's info."
    return self._request("GET", "v0/users/me")


# %% ../nbs/00_core.ipynb 16
@patch
def balance(self: Fewsats):
    "Retrieve the balance of the user's wallet."
    return self._request("GET", "v0/wallets")


# %% ../nbs/00_core.ipynb 19
@patch
def payment_methods(self: Fewsats) -> List[Dict[str, Any]]:
    "Retrieve the user's payment methods, raises an exception for error status codes."
    return self._request("GET", "v0/stripe/payment-methods")


# %% ../nbs/00_core.ipynb 23
@patch
def _preview_payment(self: Fewsats,
                    amount: str): # The amount in USD cents
    "Simulates a purchase, raises an exception for error status codes."
    assert amount.isdigit()
    return self._request("POST", "v0/l402/preview/purchase/amount", json={"amount_usd": amount})


# %% ../nbs/00_core.ipynb 26
@patch
def create_offers(self:Fewsats,
                 offers:List[Dict[str,Any]], # List of offer objects following OfferCreateV0 schema
) -> dict:
    "Create offers for L402 payment server"
    return self._request("POST", "v0/l402/offers", json={"offers": offers})

# %% ../nbs/00_core.ipynb 29
@patch
def get_payment_details(self:Fewsats,
                       payment_request_url:str,
                       offer_id:str,
                       payment_method:str,
                       payment_context_token:str,
                       ) -> dict:
    data = {"offer_id": offer_id, "payment_method": payment_method, "payment_context_token": payment_context_token}
    return httpx.post(payment_request_url, json=data)


# %% ../nbs/00_core.ipynb 33
@patch
def get_payment_status(self:Fewsats,
                       payment_context_token:str,
                       ) -> dict:
    return self._request("GET", f"v0/l402/payment-status?payment_context_token={payment_context_token}")

# %% ../nbs/00_core.ipynb 35
@patch
def set_webhook(self:Fewsats,
                       webhook_url:str,
                       ) -> dict:
    return self._request("POST", f"v0/users/webhook/set", json={"webhook_url": webhook_url})

# %% ../nbs/00_core.ipynb 38
@patch
def pay_lightning(self: Fewsats, 
                  invoice: str, # lightning invoice
                  amount: int, # amount in cents
                  currency: str = "usd", # currency
                  description: str = "" ): # description of the payment 
    "Pay for a lightning invoice"
    data = {
        "invoice": invoice,
        "amount": amount,
        "currency": currency,
        "description": description
    }
    return self._request("POST", "v0/l402/purchases/lightning", json=data)

# %% ../nbs/00_core.ipynb 42
class Offer(BasicRepr):
    "Represents a single L402 offer"
    def __init__(self, 
                 offer_id: str,
                 amount: int,
                 currency: str,
                 description: str,
                 title: str,
                 payment_methods: List[str] = None,
                 type: str = "one-off"): 
        store_attr()

    def __repr__(self):
        return f"Offer: {self.title}\nID: {self.offer_id}\nAmount: {self.amount/100} {self.currency}\nDescription: {self.description}"
    
    @classmethod
    def from_dict(cls, d: Dict[str, Any]) -> 'Offer':
        "Create an Offer from a dictionary"
        return cls(**d)

class L402Offers(BasicRepr):
    "Represents the complete L402 offers schema"
    def __init__(self, 
                 offers: List[Offer],
                 payment_context_token: str,
                 payment_request_url: str,
                 version: str): 
        store_attr()
    
    def __repr__(self):
        offers_str = "\n".join([f"- {o.title} ({o.amount/100} {o.currency})" for o in self.offers])
        return f"L402 Offers:\n{offers_str}\nPayment URL: {self.payment_request_url}\nContext Token: {self.payment_context_token}"
    
    def as_dict(self) -> Dict[str, Any]:
        "Convert to dictionary format for API usage"
        return {
            'offers': [vars(o) for o in self.offers],
            'payment_context_token': self.payment_context_token,
            'payment_request_url': self.payment_request_url,
            'version': self.version
        }
    
    @classmethod
    def from_dict(cls, d: Dict[str, Any]) -> 'L402Offers':
        "Create an L402Offers object from a dictionary"
        offers = [Offer.from_dict(o) for o in d['offers']]
        return cls(
            offers=offers,
            payment_context_token=d['payment_context_token'],
            payment_request_url=d['payment_request_url'],
            version=d['version']
        )


# %% ../nbs/00_core.ipynb 46
@patch
def pay_offer(self:Fewsats,
        offer_id : str, # the offer id to pay for
        l402_offer: L402Offers, # a dictionary containing L402 offers
) -> dict: # payment status response
    """Pays an offer_id from the l402_offers. This tools requires the LLM caller to 
    support custom classes as parameters like Claudette does.

    Returns payment status response"""
    offer_dict = l402_offer.as_dict()
    data = {"offer_id": offer_id, **offer_dict}
    return self._request("POST", "v0/l402/purchases/from-offer", json=data)


# %% ../nbs/00_core.ipynb 50
@patch
def pay_offer_str(self:Fewsats,
        offer_id : str, # the offer id to pay for
        l402_offer: str, # JSON string containing L402 offers
) -> dict: # payment status response
    """Pays an offer_id from the l402_offers.

    The l402_offer parameter must be a JSON string with this structure:
    {
        'offers': [
            {
                'offer_id': 'test_offer_2',  # String identifier for the offer
                'amount': 1,                 # Numeric cost value
                'currency': 'usd',           # Currency code
                'description': 'Test offer', # Text description
                'title': 'Test Package'      # Title of the package
            }
        ],
        'payment_context_token': '60a8e027-8b8b-4ccf-b2b9-380ed0930283',  # Payment context token
        'payment_request_url': 'https://api.fewsats.com/v0/l402/payment-request',  # Payment URL
        'version': '0.2.2'  # API version
    }

    Returns payment status response"""
    # Parse JSON string to dictionary
    try:
        offer_data = json.loads(l402_offer)
        L402Offers.from_dict(offer_data) # we don't care about the return value, just validating the json input
    except json.JSONDecodeError:
        raise ValueError("Invalid JSON string provided for l402_offer")
    
    # Create payload with offer_id
    data = {"offer_id": offer_id, **offer_data}
    
    return self._request("POST", "v0/l402/purchases/from-offer", timeout=20, json=data)

# %% ../nbs/00_core.ipynb 54
@patch
def payment_info(self:Fewsats,
                  pid:str): # purchase id
    "Retrieve the details of a payment."
    return self._request("GET", f"v0/l402/outgoing-payments/{pid}")

# %% ../nbs/00_core.ipynb 57
@patch
def as_tools(self:Fewsats):
    "Return list of available tools for AI agents"
    return [
        self.me,
        self.balance,
        self.payment_methods,
        self.pay_offer_str,
        self.payment_info,
    ]
